@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: eval.c,v 1.146 2017/05/05 06:09:32 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - eval.c */
 
 /*[
@@ -68,6 +64,12 @@ struct udft_entry *first_udf = NULL;
 /* pointer to first udv users can delete */
 struct udvt_entry **udv_user_head;
 
+/* Various abnormal conditions during evaluation of an action table
+ * (the stored form of an expression) are signalled by setting
+ * undefined = TRUE.
+ * NB:  A test for  "if (undefined)"  is only valid immediately
+ * following a call to evaluate_at() or eval_link_function().
+ */
 TBOOLEAN undefined;
 
 /* The stack this operates on */
@@ -405,7 +407,7 @@ struct value *
 Gstring(struct value *a, char *s)
 {
     a->type = STRING;
-    a->v.string_val = s;
+    a->v.string_val = s ? s : strdup("");
     return (a);
 }
 
@@ -687,7 +689,12 @@ execute_at(struct at_type *at_ptr)
 void
 evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 {
+    /* A test for if (undefined) is allowed only immediately following
+     * evalute_at() or eval_link_function().  Both must clear it on entry
+     * so that the value on return reflects what really happened.
+     */
     undefined = FALSE;
+
     errno = 0;
     reset_stack();
 
@@ -715,7 +722,8 @@ evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 	 * E.g. load_one_range()
 	 */
 	val_ptr->type = NOTDEFINED;
-	int_error(NO_CARET, "evaluate_at: unsupported array operation");
+	if (!string_result_only)
+	    int_error(NO_CARET, "evaluate_at: unsupported array operation");
     }
 }
 
