@@ -331,18 +331,17 @@ load_file(FILE *fp, char *name, int calltype)
     int start, left;
     int more;
     int stop = FALSE;
+    udvt_entry *gpval_lineno = NULL;
+
+    if (fp == (FILE *) NULL)
+	int_error(NO_CARET, "Cannot open script file '%s'", name);
 
     /* Provide a user-visible copy of the current line number in the input file */
-    udvt_entry *gpval_lineno = add_udv_by_name("GPVAL_LINENO");
+    gpval_lineno = add_udv_by_name("GPVAL_LINENO");
     Ginteger(&gpval_lineno->udv_value, 0);
 
     lf_push(fp, name, NULL); /* save state for errors and recursion */
 
-    if (fp == (FILE *) NULL) {
-	int_error(NO_CARET, "Cannot open script file '%s'", name);
-	return; /* won't actually reach here */
-    }
-
     if (fp == stdin) {
 	/* DBT 10-6-98  go interactive if "-" named as load file */
 	interactive = TRUE;
@@ -374,7 +373,9 @@ load_file(FILE *fp, char *name, int calltype)
 	    } else {
 		inline_num++;
 		gpval_lineno->udv_value.v.int_val = inline_num;	/* User visible copy */
-		len = strlen(gp_input_line) - 1;
+		if ((len = strlen(gp_input_line)) == 0)
+		    continue;
+		--len;
 		if (gp_input_line[len] == '\n') {	/* remove any newline */
 		    gp_input_line[len] = '\0';
 		    /* Look, len was 1-1 = 0 before, take care here! */
