@@ -70,6 +70,7 @@ public:
 	virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* event);
 	virtual void wheelEvent(QGraphicsSceneWheelEvent* event);
 	virtual void keyPressEvent(QKeyEvent* event);
+	virtual void keyReleaseEvent(QKeyEvent* event);
 	void processEvent(QtGnuplotEventType type, QDataStream& in);
 
 private:
@@ -101,6 +102,8 @@ private:
 	QPointF m_zoomBoxCorner;
 	double  m_currentPointSize;
 	double  m_textAngle;
+	double  m_currentBoxRotation;
+	QPoint  m_currentBoxOrigin;
 	QPoint  m_textOffset;
 	double  m_currentZ;
 	QTime   m_watches[4];
@@ -108,6 +111,7 @@ private:
 	bool    m_inKeySample;
 	bool    m_preserve_visibility;
 	bool	m_inTextBox;
+	int	m_currentFillStyle;
 	QRectF	m_currentTextBox;
 	QPointF m_textMargin;
 	QList<QGraphicsItem*> m_currentGroup;
@@ -128,9 +132,10 @@ private:
 	QList<QtGnuplotKeybox> m_key_boxes;
 	QString m_currentHypertext;
 	QList<QGraphicsItem*> m_hypertextList;
+	QGraphicsPixmapItem* m_hyperimage;
 
 	// Axis scales
-	bool   m_axisValid[4];
+	bool   m_axisValid[5];	// x, y, x2, y2, z (indicates 3D plot)
 	double m_axisMin  [4];
 	double m_axisLower[4];
 	double m_axisScale[4];
