@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: readline.c,v 1.58.2.1 2014/01/11 16:22:13 markisch Exp $"); }
-#endif
-
 /* GNUPLOT - readline.c */
 
 /*[
@@ -61,52 +57,74 @@ static char *RCSid() { return RCSid("$Id: readline.c,v 1.58.2.1 2014/01/11 16:22
 #ifdef HAVE_WCHAR_H
 #include <wchar.h>
 #endif
+#ifdef WGP_CONSOLE
+#include "win/winmain.h"
+#endif
 
+/*
+ * adaptor routine for gnu libreadline
+ * to allow multiplexing terminal and mouse input
+ */
 #if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
-#if defined(HAVE_LIBEDITLINE)
 int
-#else
-static int
-#endif
-#if defined(HAVE_LIBEDITLINE)
-getc_wrapper(FILE* fp /* is NULL, supplied by libedit */)
-#else
-getc_wrapper(FILE* fp /* should be stdin, supplied by readline */)
-#endif
+getc_wrapper(FILE* fp)
 {
     int c;
 
     while (1) {
+	errno = 0;
 #ifdef USE_MOUSE
 	if (term && term->waitforinput && interactive) {
-	    c = term->waitforinput();
+	    c = term->waitforinput(0);
 	}
 	else
 #endif
+#if defined(WGP_CONSOLE)
+	    c = ConsoleGetch();
+#else
 	if (fp)
 	    c = getc(fp);
 	else
 	    c = getchar(); /* HAVE_LIBEDITLINE */
 	if (c == EOF && errno == EINTR)
 	    continue;
+#endif
 	return c;
     }
 }
 #endif /* HAVE_LIBREADLINE || HAVE_LIBEDITLINE */
 
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE) || defined(READLINE)
-char*
-readline_ipc(const char* prompt)
+#if defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_SIGNAL_HANDLER)
+/*
+ * The signal handler of libreadline sets a flag when SIGTSTP is received
+ * but does not suspend until this flag is checked by other library
+ * routines.  Since gnuplot's term->waitforinput() + getc_wrapper() 
+ * replace these other routines, we must do the test and suspend ourselves.
+ */
+void
+wrap_readline_signal_handler()
 {
-#if defined(PIPE_IPC) && defined(HAVE_LIBREADLINE)
-    rl_getc_function = getc_wrapper;
-#endif
-    return readline((const char*) prompt);
+    int sig;
+
+    /* FIXME:	At the moment, there is no portable way to invoke the
+	    signal handler. */
+    extern void _rl_signal_handler(int);
+
+# ifdef HAVE_READLINE_PENDING_SIGNAL
+    sig = rl_pending_signal();
+# else
+    /* XXX: We assume all versions of readline have this... */
+    extern int volatile _rl_caught_signal;
+    sig = _rl_caught_signal;
+# endif
+
+    if (sig) _rl_signal_handler(sig);
 }
-#endif  /* HAVE_LIBREADLINE || HAVE_LIBEDITLINE || READLINE */
+
+#endif /* defined(HAVE_LIBREADLINE) && defined(HAVE_READLINE_SIGNAL_HANDLER) */
 
 
-#if defined(READLINE) && !(defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE))
+#ifdef READLINE
 
 /* This is a small portable version of GNU's readline that does not require
  * any terminal capabilities except backspace and space overwrites a character.
@@ -126,11 +144,14 @@ readline_ipc(const char* prompt)
  * ^N moves forward through history
  * ^H deletes the previous character
  * ^D deletes the current character, or EOF if line is empty
- * ^L/^R redraw line in case it gets trashed
+ * ^L redraw line in case it gets trashed
  * ^U kills the entire line
  * ^W deletes previous full or partial word
+ * ^V disables interpretation of the following key
  * LF and CR return the entire line regardless of the cursor postition
  * DEL deletes previous or current character (configuration dependent)
+ * TAB will perform filename completion
+ * ^R start a backward-search of the history
  * EOF with an empty line returns (char *)NULL
  *
  * all other characters are ignored
@@ -150,7 +171,7 @@ readline_ipc(const char* prompt)
 # endif
 #endif /* not HAVE_TERMIOS_H && HAVE_TCGETATTR */
 
-#if !defined(MSDOS) && !defined(_Windows)
+#if !defined(MSDOS) && !defined(_WIN32)
 
 /*
  * Set up structures using the proper include file
@@ -238,23 +259,28 @@ static int term_set = 0;	/* =1 if rl_termio set */
 static int ansi_getc __PROTO((void));
 #define DEL_ERASES_CURRENT_CHAR
 
-#else /* MSDOS or _Windows */
+#else /* MSDOS or _WIN32 */
 
-# ifdef _Windows
+# ifdef _WIN32
 #  include <windows.h>
-#  include "win/wtext.h"
 #  include "win/winmain.h"
+#  include "win/wcommon.h"
 #  define TEXTUSER 0xf1
 #  define TEXTGNUPLOT 0xf0
 #  ifdef WGP_CONSOLE
 #   define special_getc() win_getch()
-static char win_getch __PROTO((void));
+static int win_getch(void);
 #  else
+    /* The wgnuplot text window will suppress intermediate
+       screen updates in 'suspend' mode and only redraw the 
+       input line after 'resume'. */
+#   define SUSPENDOUTPUT TextSuspend(&textwin)
+#   define RESUMEOUTPUT TextResume(&textwin)
 #   define special_getc() msdos_getch()
+static int msdos_getch(void);
 #  endif /* WGP_CONSOLE */
-static char msdos_getch __PROTO((void));	/* HBB 980308: PROTO'ed it */
 #  define DEL_ERASES_CURRENT_CHAR
-# endif				/* _Windows */
+# endif				/* _WIN32 */
 
 # if defined(MSDOS)
 /* MSDOS specific stuff */
@@ -265,36 +291,47 @@ static char msdos_getch __PROTO((void));	/* HBB 980308: PROTO'ed it */
 #   include <conio.h>
 #  endif			/* __EMX__ */
 #  define special_getc() msdos_getch()
-static char msdos_getch();
+static int msdos_getch();
 #  define DEL_ERASES_CURRENT_CHAR
 # endif				/* MSDOS */
 
-#endif /* MSDOS or _Windows */
+#endif /* MSDOS or _WIN32 */
 
 #ifdef OS2
 # if defined( special_getc )
 #  undef special_getc()
 # endif				/* special_getc */
 # define special_getc() os2_getch()
-static char msdos_getch __PROTO((void));	/* HBB 980308: PROTO'ed it */
-static char os2_getch __PROTO((void));
+static int msdos_getch(void);
+static int os2_getch(void);
 #  define DEL_ERASES_CURRENT_CHAR
 #endif /* OS2 */
 
 
 /* initial size and increment of input line length */
 #define MAXBUF	1024
-#define BACKSPACE 0x08   /* ^H */
+#define BACKSPACE '\b'   /* ^H */
 #define SPACE	' '
 #define NEWLINE	'\n'
 
 #define MAX_COMPLETIONS 50
 
+#ifndef SUSPENDOUTPUT
+#define SUSPENDOUTPUT
+#define RESUMEOUTPUT
+#endif
+
 static char *cur_line;		/* current contents of the line */
 static size_t line_len = 0;
 static size_t cur_pos = 0;	/* current position of the cursor */
 static size_t max_pos = 0;	/* maximum character position */
 
+static TBOOLEAN search_mode = FALSE;
+static const char search_prompt[] = "search '";
+static const char search_prompt2[] = "': ";
+static struct hist * search_result = NULL;
+static int search_result_width = 0;	/* on-screen width of the search result */
+
 static void fix_line __PROTO((void));
 static void redraw_line __PROTO((const char *prompt));
 static void clear_line __PROTO((const char *prompt));
@@ -311,10 +348,18 @@ static void step_forward __PROTO((void));
 static void delete_forward __PROTO((void));
 static void delete_backward __PROTO((void));
 static int char_seqlen __PROTO((void));
-#if defined(HAVE_DIRENT_H) || defined(WIN32)
+#if defined(HAVE_DIRENT_H) || defined(_WIN32)
 static char *fn_completion(size_t anchor_pos, int direction);
 static void tab_completion(TBOOLEAN forward);
 #endif
+static void switch_prompt(const char * old_prompt, const char * new_prompt);
+static int do_search(int dir);
+static void print_search_result(const struct hist * result);
+
+#ifndef _WIN32
+static int mbwidth(char *c);
+#endif
+static int strwidth(const char * str);
 
 /* user_putc and user_puts should be used in the place of
  * fputc(ch,stderr) and fputs(str,stderr) for all output
@@ -325,16 +370,12 @@ static int
 user_putc(int ch)
 {
     int rv;
-#ifdef _Windows
-#ifndef WGP_CONSOLE
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
     TextAttr(&textwin, TEXTUSER);
-#endif
 #endif
     rv = fputc(ch, stderr);
-#ifdef _Windows
-#ifndef WGP_CONSOLE
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
     TextAttr(&textwin, TEXTGNUPLOT);
-#endif
 #endif
     return rv;
 }
@@ -343,21 +384,18 @@ static int
 user_puts(char *str)
 {
     int rv;
-#ifdef _Windows
-#ifndef WGP_CONSOLE
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
     TextAttr(&textwin, TEXTUSER);
-#endif
 #endif
     rv = fputs(str, stderr);
-#ifdef _Windows
-#ifndef WGP_CONSOLE
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
     TextAttr(&textwin, TEXTGNUPLOT);
-#endif
 #endif
     return rv;
 }
 
 
+#if !defined(_WIN32)
 /* EAM FIXME
  * This test is intended to determine if the current character, of which
  * we have only seen the first byte so far, will require twice the width
@@ -381,36 +419,81 @@ mbwidth(char *c)
 	return ((unsigned char)(*c) >= 0xe3 ? 2 : 1);
 #endif
     }
+
+    case S_ENC_SJIS: {
+	/* Assume all double-byte characters have double-width. */
+	return is_sjis_lead_byte(*c) ? 2 : 1;
+    }
+
     default:
-        return 1;
+	return 1;
     }
 }
+#endif
 
+static int
+strwidth(const char * str)
+{
+#if !defined(_WIN32)
+    int width = 0;
+    int i = 0;
+
+    switch (encoding) {
+    case S_ENC_UTF8: {
+	char ch = str[i];
+	if ((ch & 0xE0) == 0xC0) {
+	    i += 1;
+	} else if ((ch & 0xF0) == 0xE0) {
+	    i += 2;
+	} else if ((ch & 0xF8) == 0xF0) {
+	    i += 3;
+	}
+	width += mbwidth(&ch);
+    }
+    case S_ENC_SJIS:
+	/* Assume all double-byte characters have double-width. */
+	width = gp_strlen(str);
+	break;
+    default:
+	width = strlen(str);
+    }
+    return width;
+#else
+    /* double width characters are handled in the backend */
+    return gp_strlen(str);
+#endif
+}
 
 static int
 isdoublewidth(size_t pos)
 {
+#if defined(_WIN32)
+    /* double width characters are handled in the backend */
+    return FALSE;
+#else
     return mbwidth(cur_line + pos) > 1;
+#endif
 }
 
-
 /*
  * Determine length of multi-byte sequence starting at current position
  */
 static int
 char_seqlen()
 {
-    int i;
-
     switch (encoding) {
 
-    case S_ENC_UTF8:
-	i = cur_pos;
+    case S_ENC_UTF8: {
+	int i = cur_pos;
 	do {i++;}
-	    while ((cur_line[i] & 0xc0) != 0xc0
-	        && (cur_line[i] & 0x80) != 0
-		&& i < max_pos);
+	while (((cur_line[i] & 0xc0) != 0xc0)
+	       && ((cur_line[i] & 0x80) != 0)
+	       && (i < max_pos));
 	return (i - cur_pos);
+    }
+
+    case S_ENC_SJIS:
+	return is_sjis_lead_byte(cur_line[cur_pos]) ? 2 : 1;
 
     default:
 	return 1;
@@ -418,29 +501,50 @@ char_seqlen()
 }
 
 /*
- * Back up over one multi-byte UTF-8 character sequence immediately preceding
+ * Back up over one multi-byte character sequence immediately preceding
  * the current position.  Non-destructive.  Affects both cur_pos and screen cursor.
  */
 static int
 backspace()
 {
-    int seqlen;
-
     switch (encoding) {
 
-    case S_ENC_UTF8:
-	seqlen = 0;
-	do {cur_pos--; seqlen++;}
-	    while ((cur_line[cur_pos] & 0xc0) != 0xc0
-	        && (cur_line[cur_pos] & 0x80) != 0
-		&& cur_pos > 0);
+    case S_ENC_UTF8: {
+	int seqlen = 0;
+	do {
+	    cur_pos--;
+	    seqlen++;
+	} while (((cur_line[cur_pos] & 0xc0) != 0xc0)
+	         && ((cur_line[cur_pos] & 0x80) != 0)
+		 && (cur_pos > 0)
+	        );
 
-	if ((cur_line[cur_pos] & 0xc0) == 0xc0
-	||  isprint(cur_line[cur_pos]))
+	if (   ((cur_line[cur_pos] & 0xc0) == 0xc0)
+	    || isprint((unsigned char)cur_line[cur_pos])
+	   )
 	    user_putc(BACKSPACE);
 	if (isdoublewidth(cur_pos))
 	    user_putc(BACKSPACE);
 	return seqlen;
+    }
+
+    case S_ENC_SJIS: {
+	/* With S-JIS you cannot always determine if a byte is a single byte or part
+	   of a double-byte sequence by looking of an arbitrary byte in a string.
+	   Always test from the start of the string instead.
+	*/
+	int i;
+        int seqlen = 1;
+
+	for (i = 0; i < cur_pos; i += seqlen) {
+	    seqlen = is_sjis_lead_byte(cur_line[i]) ? 2 : 1;
+	}
+	cur_pos -= seqlen;
+	user_putc(BACKSPACE);
+	if (isdoublewidth(cur_pos))
+	    user_putc(BACKSPACE);
+	return seqlen;
+    }
 
     default:
 	cur_pos--;
@@ -462,8 +566,9 @@ step_forward()
     switch (encoding) {
 
     case S_ENC_UTF8:
+    case S_ENC_SJIS:
 	seqlen = char_seqlen();
-	for (i=0; i<seqlen; i++)
+	for (i = 0; i < seqlen; i++)
 	    user_putc(cur_line[cur_pos++]);
 	break;
 
@@ -512,8 +617,8 @@ extend_cur_line()
 {
     char *new_line;
 
-    /* extent input line length */
-    new_line = gp_realloc(cur_line, line_len + MAXBUF, NULL);
+    /* extend input line length */
+    new_line = (char *) gp_realloc(cur_line, line_len + MAXBUF, NULL);
     if (!new_line) {
 	reset_termio();
 	int_error(NO_CARET, "Can't extend readline length");
@@ -524,7 +629,7 @@ extend_cur_line()
 }
 
 
-#if defined(HAVE_DIRENT_H) || defined(WIN32)
+#if defined(HAVE_DIRENT_H) || defined(_WIN32)
 static char *
 fn_completion(size_t anchor_pos, int direction)
 {
@@ -554,21 +659,26 @@ fn_completion(size_t anchor_pos, int direction)
 	start = cur_line + anchor_pos;
 	if (anchor_pos > 0) {
 	    /* first, look for a quote to start the string */
-	    for ( ; start > cur_line; start--) {
+	    for ( ; start >= cur_line; start--) {
 	        if ((*start == '"') || (*start == '\'')) {
 		    start++;
+		    /* handle pipe commands */
+		    if ((*start == '<') || (*start == '|'))
+			start++;
 		    break;
 		}
 	    }
-	    /* if not found, search for a space instead */
-	    if (start == cur_line) {
-		for (start = cur_line + anchor_pos ; start > cur_line; start--) {
+	    /* if not found, search for a space or a system command '!' instead */
+	    if (start <= cur_line) {
+		for (start = cur_line + anchor_pos; start >= cur_line; start--) {
 		    if ((*start == ' ') || (*start == '!')) {
 			start++;
 			break;
 		    }
 		}
 	    }
+	    if (start < cur_line)
+		start = cur_line;
 
 	    path = strndup(start, cur_line - start + anchor_pos);
 	    gp_expand_tilde(&path);
@@ -671,6 +781,7 @@ tab_completion(TBOOLEAN forward)
 	while (max_pos + completion_len - last_completion_len + 1 > line_len)
 	    extend_cur_line();
 
+    SUSPENDOUTPUT;
     /* erase from last_tab_pos to eol */
     while (cur_pos > last_tab_pos)
 	backspace();
@@ -699,13 +810,14 @@ tab_completion(TBOOLEAN forward)
 	user_putc(cur_line[last_tab_pos+i]);
     cur_pos += completion_len;
     fix_line();
+    RESUMEOUTPUT;
 
     /* remember this completion */
     last_tab_pos  = cur_pos - completion_len;
     last_completion_len = completion_len;
 }
 
-#endif /* HAVE_DIRENT_H || WIN32 */
+#endif /* HAVE_DIRENT_H || _WIN32 */
 
 
 char *
@@ -713,14 +825,15 @@ readline(const char *prompt)
 {
     int cur_char;
     char *new_line;
-
+    TBOOLEAN next_verbatim = FALSE;
+    char *prev_line;
 
     /* start with a string of MAXBUF chars */
     if (line_len != 0) {
 	free(cur_line);
 	line_len = 0;
     }
-    cur_line = gp_alloc(MAXBUF, "readline");
+    cur_line = (char *) gp_alloc(MAXBUF, "readline");
     line_len = MAXBUF;
 
     /* set the termio so we can do our own input processing */
@@ -731,7 +844,12 @@ readline(const char *prompt)
     cur_line[0] = '\0';
     cur_pos = 0;
     max_pos = 0;
-    cur_entry = NULL;
+
+    /* move to end of history */
+    while (next_history());
+
+    /* init global variables */
+    search_mode = FALSE;
 
     /* get characters */
     for (;;) {
@@ -741,9 +859,10 @@ readline(const char *prompt)
 	/* Accumulate ascii (7bit) printable characters
 	 * and all leading 8bit characters.
 	 */
-	if ((isprint(cur_char)
-	      || (((cur_char & 0x80) != 0) && (cur_char != EOF)))
-	    && (cur_char != 0x09) /* TAB is a printable character in some locales */
+	if (((isprint(cur_char)
+	      || (((cur_char & 0x80) != 0) && (cur_char != EOF))
+	     ) && (cur_char != '\t')) /* TAB is a printable character in some locales */
+	    || next_verbatim
 	    ) {
 	    size_t i;
 
@@ -763,24 +882,76 @@ readline(const char *prompt)
 	    if (cur_pos < max_pos) {
 		switch (encoding) {
 		case S_ENC_UTF8:
-		    if ((cur_char & 0xc0) == 0)
+		    if ((cur_char & 0xc0) == 0) {
+			next_verbatim = FALSE;
 			fix_line(); /* Normal ascii character */
-		    else if ((cur_char & 0xc0) == 0xc0)
+		    } else if ((cur_char & 0xc0) == 0xc0) {
 			; /* start of a multibyte sequence. */
-		    else if (((cur_char & 0xc0) == 0x80) &&
-			 ((unsigned char)(cur_line[cur_pos-2]) >= 0xe0))
+		    } else if (((cur_char & 0xc0) == 0x80) &&
+			 ((unsigned char)(cur_line[cur_pos-2]) >= 0xe0)) {
 			; /* second byte of a >2 byte sequence */
-		    else {
+		    } else {
 			/* Last char of multi-byte sequence */
+			next_verbatim = FALSE;
 			fix_line();
 		    }
 		    break;
+
+		case S_ENC_SJIS: {
+		    /* S-JIS requires a state variable */
+		    static int mbwait = 0;
+
+		    if (mbwait == 0) {
+		        if (!is_sjis_lead_byte(cur_char)) {
+			    /* single-byte character */
+			    next_verbatim = FALSE;
+			    fix_line();
+			} else {
+			    /* first byte of a double-byte sequence */
+			    ;
+			}
+		    } else {
+			/* second byte of a double-byte sequence */
+			mbwait = 0;
+			next_verbatim = FALSE;
+			fix_line();
+		    }
+		}
+
 		default:
+		    next_verbatim = FALSE;
 		    fix_line();
 		    break;
 		}
+	    } else {
+		static int mbwait = 0;
+
+		next_verbatim = FALSE;
+		if (search_mode) {
+		    /* Only update the search at the end of a multi-byte sequence. */
+		    if (mbwait == 0) {
+			if (encoding == S_ENC_SJIS)
+			    mbwait = is_sjis_lead_byte(cur_char) ? 1 : 0;
+			if (encoding == S_ENC_UTF8) {
+			    char ch = cur_char;
+			    if (ch & 0x80)
+				while ((ch = (ch << 1)) & 0x80)
+				    mbwait++;
+			}
+		    } else {
+			mbwait--;
+		    }
+		    if (!mbwait)
+			do_search(-1);
+		}
 	    }
 
+	/* ignore special characters in search_mode */
+	} else if (!search_mode) {
+
+	if (0) {
+	    ;
+
 	/* else interpret unix terminal driver characters */
 #ifdef VERASE
 	} else if (cur_char == term_chars[VERASE]) {	/* ^H */
@@ -803,9 +974,11 @@ readline(const char *prompt)
 	    delete_previous_word();
 #endif /* VWERASE */
 #ifdef VREPRINT
+#if 0 /* conflict with reverse-search */
 	} else if (cur_char == term_chars[VREPRINT]) {	/* ^R? */
 	    putc(NEWLINE, stderr);	/* go to a fresh line */
 	    redraw_line(prompt);
+#endif
 #endif /* VREPRINT */
 #ifdef VSUSP
 	} else if (cur_char == term_chars[VSUSP]) {
@@ -844,7 +1017,7 @@ readline(const char *prompt)
 		    step_forward();
 		}
 		break;
-#if defined(HAVE_DIRENT_H) || defined(WIN32)
+#if defined(HAVE_DIRENT_H) || defined(_WIN32)
 	    case 011:		/* ^I / TAB */
 		tab_completion(TRUE); /* next tab completion */
 		break;
@@ -857,30 +1030,29 @@ readline(const char *prompt)
 		max_pos = cur_pos;
 		break;
 	    case 020:		/* ^P */
-		if (history != NULL) {
-		    if (cur_entry == NULL) {
-			cur_entry = history;
-			clear_line(prompt);
-			copy_line(cur_entry->line);
-		    } else if (cur_entry->prev != NULL) {
-			cur_entry = cur_entry->prev;
-			clear_line(prompt);
-			copy_line(cur_entry->line);
-		    }
+		if (previous_history() != NULL) {
+		    clear_line(prompt);
+		    copy_line(current_history()->line);
 		}
 		break;
 	    case 016:		/* ^N */
-		if (cur_entry != NULL) {
-		    cur_entry = cur_entry->next;
-		    clear_line(prompt);
-		    if (cur_entry != NULL)
-			copy_line(cur_entry->line);
-		    else
-			cur_pos = max_pos = 0;
+		clear_line(prompt);
+		if (next_history() != NULL) {
+		    copy_line(current_history()->line);
+		} else {
+		    cur_pos = max_pos = 0;
 		}
 		break;
-	    case 014:		/* ^L */
 	    case 022:		/* ^R */
+		prev_line = strdup(cur_line);
+		switch_prompt(prompt, search_prompt);
+		while (next_history()); /* seek to end of history */
+		search_result = NULL;
+		search_result_width = 0;
+		search_mode = TRUE;
+		print_search_result(NULL);
+		break;
+	    case 014:		/* ^L */
 		putc(NEWLINE, stderr);	/* go to a fresh line */
 		redraw_line(prompt);
 		break;
@@ -892,9 +1064,10 @@ readline(const char *prompt)
 		delete_backward();
 		break;
 	    case 004:		/* ^D */
-		if (max_pos == 0) {
+		/* Also catch asynchronous termination signal on Windows */
+		if (max_pos == 0 && terminate_flag) {
 		    reset_termio();
-		    return ((char *) NULL);
+		    return NULL;
 		}
 		/* intentionally omitting break */
 #ifdef DEL_ERASES_CURRENT_CHAR
@@ -906,6 +1079,9 @@ readline(const char *prompt)
 	    case 025:		/* ^U */
 		clear_line(prompt);
 		break;
+	    case 026:		/* ^V */
+		next_verbatim = TRUE;
+		break;
 	    case 027:		/* ^W */
 		delete_previous_word();
 		break;
@@ -937,10 +1113,136 @@ readline(const char *prompt)
 	    default:
 		break;
 	    }
+	} 
+
+	} else {  /* search-mode */
+#ifdef VERASE
+	    if (cur_char == term_chars[VERASE]) {	/* ^H */
+		delete_backward();
+		do_search(-1);
+	    } else 
+#endif /* VERASE */
+	    {
+	    switch (cur_char) {
+	    case 022:		/* ^R */
+		/* search next */
+		previous_history();
+		if (do_search(-1) == -1)
+		    next_history();
+		break;
+	    case 023:		/* ^S */
+		/* search previous */
+		next_history();
+		if (do_search(1) == -1)
+		    previous_history();
+		break;
+		break;
+	    case '\n':		/* ^J */
+	    case '\r':		/* ^M */
+		/* accept */
+		switch_prompt(search_prompt, prompt);
+		if (search_result != NULL)
+		    copy_line(search_result->line);
+		free(prev_line);
+		search_result_width = 0;
+		search_mode = FALSE;
+		break;
+#ifndef DEL_ERASES_CURRENT_CHAR
+	    case 0177:		/* DEL */
+	    /* FIXME: conflict! */
+	    //case 023:		/* Re-mapped from CSI~3 in ansi_getc() */
+#endif
+	    case 010:		/* ^H */
+		delete_backward();
+		do_search(1);
+		break;
+	    default:
+		/* abort, restore previous input line */
+		switch_prompt(search_prompt, prompt);
+		copy_line(prev_line);
+		free(prev_line);
+		search_result_width = 0;
+		search_mode = FALSE;
+		break;
+	    }
+	    }
 	}
     }
 }
 
+
+static int
+do_search(int dir)
+{
+    int ret = -1;
+
+    if ((ret = history_search(cur_line, dir)) != -1)
+	search_result = current_history();
+    print_search_result(search_result);
+    return ret;
+}
+
+
+void
+print_search_result(const struct hist * result)
+{
+    int i, width = 0;
+
+    SUSPENDOUTPUT;
+    fputs(search_prompt2, stderr);
+    if (result != NULL && result->line != NULL) {
+	fputs(result->line, stderr);
+	width = strwidth(result->line);
+    }
+
+    /* overwrite previous search result, and the line might
+       just have gotten 1 double-width character shorter */
+    for (i = 0; i < search_result_width - width + 2; i++)
+	putc(SPACE, stderr);
+    for (i = 0; i < search_result_width - width + 2; i++)
+	putc(BACKSPACE, stderr);
+    search_result_width = width;
+
+    /* restore cursor position */
+    for (i = 0; i < width; i++)
+	putc(BACKSPACE, stderr);
+    for (i = 0; i < strlen(search_prompt2); i++)
+	putc(BACKSPACE, stderr);
+    RESUMEOUTPUT;
+}
+
+
+static void
+switch_prompt(const char * old_prompt, const char * new_prompt)
+{
+    int i, len;
+
+    SUSPENDOUTPUT;
+
+    /* clear search results (if any) */
+    if (search_mode) {
+	for (i = 0; i < search_result_width + strlen(search_prompt2); i++)
+	    user_putc(SPACE);
+	for (i = 0; i < search_result_width + strlen(search_prompt2); i++)
+	    user_putc(BACKSPACE);
+    }
+
+    /* clear current line */
+    clear_line(old_prompt);
+    putc('\r', stderr);
+    fputs(new_prompt, stderr);
+    cur_pos = 0;
+    
+    /* erase remainder of previous prompt */
+    len = GPMAX((int)strlen(old_prompt) - (int)strlen(new_prompt), 0);
+    for (i = 0; i < len; i++)
+	user_putc(SPACE);
+    for (i = 0; i < len; i++)
+	user_putc(BACKSPACE);
+    RESUMEOUTPUT;
+}
+
+
 /* Fix up the line from cur_pos to max_pos.
  * Does not need any terminal capabilities except backspace,
  * and space overwrites a character
@@ -950,6 +1252,7 @@ fix_line()
 {
     size_t i;
 
+    SUSPENDOUTPUT;
     /* write tail of string */
     for (i = cur_pos; i < max_pos; i++)
 	user_putc(cur_line[i]);
@@ -960,6 +1263,12 @@ fix_line()
      */
     user_putc(SPACE);
     user_putc(SPACE);
+    if (search_mode) {
+	for (i = 0; i < search_result_width; i++)
+	    user_putc(SPACE);
+	for (i = 0; i < search_result_width; i++)
+	    user_putc(BACKSPACE);
+    }
     user_putc(BACKSPACE);
     user_putc(BACKSPACE);
 
@@ -967,7 +1276,7 @@ fix_line()
     i = cur_pos;
     for (cur_pos = max_pos; cur_pos > i; )
 	backspace();
-
+    RESUMEOUTPUT;
 }
 
 /* redraw the entire line, putting the cursor where it belongs */
@@ -976,6 +1285,7 @@ redraw_line(const char *prompt)
 {
     size_t i;
 
+    SUSPENDOUTPUT;
     fputs(prompt, stderr);
     user_puts(cur_line);
 
@@ -983,12 +1293,14 @@ redraw_line(const char *prompt)
     i = cur_pos;
     for (cur_pos = max_pos; cur_pos > i; )
 	backspace();
+    RESUMEOUTPUT;
 }
 
 /* clear cur_line and the screen line */
 static void
 clear_line(const char *prompt)
 {
+    SUSPENDOUTPUT;
     putc('\r', stderr);
     fputs(prompt, stderr);
     cur_pos = 0;
@@ -1005,6 +1317,7 @@ clear_line(const char *prompt)
     putc('\r', stderr);
     fputs(prompt, stderr);
     cur_pos = 0;
+    RESUMEOUTPUT;
 }
 
 /* clear to end of line and the screen end of line */
@@ -1013,6 +1326,7 @@ clear_eoline(const char *prompt)
 {
     size_t save_pos = cur_pos;
 
+    SUSPENDOUTPUT;
     while (cur_pos < max_pos) {
 	user_putc(SPACE);
 	if (isdoublewidth(cur_line[cur_pos]))
@@ -1026,6 +1340,7 @@ clear_eoline(const char *prompt)
     putc('\r', stderr);
     fputs(prompt, stderr);
     user_puts(cur_line);
+    RESUMEOUTPUT;
 }
 
 /* delete the full or partial word immediately before cursor position */
@@ -1033,6 +1348,8 @@ static void
 delete_previous_word()
 {
     size_t save_pos = cur_pos;
+
+    SUSPENDOUTPUT;
     /* skip whitespace */
     while ((cur_pos > 0) &&
 	   (cur_line[cur_pos - 1] == SPACE)) {
@@ -1066,6 +1383,7 @@ delete_previous_word()
 	max_pos = cur_pos + m;
 	fix_line();
     }
+    RESUMEOUTPUT;
 }
 
 /* copy line to cur_line, draw it and set cur_pos and max_pos */
@@ -1080,7 +1398,7 @@ copy_line(char *line)
     cur_pos = max_pos = strlen(cur_line);
 }
 
-#if !defined(MSDOS) && !defined(_Windows)
+#if !defined(MSDOS) && !defined(_WIN32)
 /* Convert ANSI arrow keys to control characters */
 static int
 ansi_getc()
@@ -1089,7 +1407,7 @@ ansi_getc()
 
 #ifdef USE_MOUSE
     if (term && term->waitforinput && interactive)
-	c = term->waitforinput();
+	c = term->waitforinput(0);
     else
 #endif
     c = getc(stdin);
@@ -1130,24 +1448,24 @@ ansi_getc()
 }
 #endif
 
-#if defined(MSDOS) || defined(_Windows) || defined(OS2)
+#if defined(MSDOS) || defined(_WIN32) || defined(OS2)
 
 #ifdef WGP_CONSOLE
-static char
+static int
 win_getch()
 {
     if (term && term->waitforinput)
-        return term->waitforinput();
+	return term->waitforinput(0);
     else
-        return ConsoleGetch();
+	return ConsoleGetch();
 }
-#endif
+#else
 
 /* Convert Arrow keystrokes to Control characters: */
-static char
+static int
 msdos_getch()
 {
-	char c;
+    int c;
 
 #ifdef DJGPP
     int ch = getkey();
@@ -1155,24 +1473,27 @@ msdos_getch()
 #elif defined (OS2)
     c = getc(stdin);
 #else /* not OS2, not DJGPP*/
-# if defined (_Windows) && defined (USE_MOUSE)
+# if defined (USE_MOUSE)
     if (term && term->waitforinput && interactive)
-	c = term->waitforinput();
+	c = term->waitforinput(0);
     else
-# endif /* not _Windows && not USE_MOUSE */
+# endif /* not USE_MOUSE */
     c = getch();
 #endif /* not DJGPP, not OS2 */
 
     if (c == 0) {
 #ifdef DJGPP
 	c = ch & 0xff;
-#else /* not DJGPP */
-# ifdef OS2
+#elif defined(OS2)
 	c = getc(stdin);
-# else				/* not OS2 */
+#else /* not OS2, not DJGPP */
+# if defined (USE_MOUSE)
+    if (term && term->waitforinput && interactive)
+	c = term->waitforinput(0);
+    else
+# endif /* not USE_MOUSE */
 	c = getch();		/* Get the extended code. */
-# endif				/* OS2 */
-#endif /* not DJGPP */
+#endif /* not DJGPP, not OS2 */
 
 	switch (c) {
 	case 75:		/* Left Arrow. */
@@ -1207,13 +1528,13 @@ msdos_getch()
     }
     return c;
 }
-
-#endif /* MSDOS || _Windows || OS2 */
+#endif /* WGP_CONSOLE */
+#endif /* MSDOS || _WIN32 || OS2 */
 
 #ifdef OS2
 /* We need to call different procedures, dependent on the
    session type: VIO/window or an (XFree86) xterm */
-static char
+static int
 os2_getch() {
   static int IsXterm = 0;
   static int init = 0;
@@ -1237,7 +1558,7 @@ os2_getch() {
 static void
 set_termio()
 {
-#if !defined(MSDOS) && !defined(_Windows)
+#if !defined(MSDOS) && !defined(_WIN32)
 /* set termio so we can do our own input processing */
 /* and save the old terminal modes so we can reset them later */
     if (term_set == 0) {
@@ -1341,13 +1662,13 @@ set_termio()
 #  endif			/* not SGTTY */
 	term_set = 1;
     }
-#endif /* not MSDOS && not _Windows */
+#endif /* not MSDOS && not _WIN32 */
 }
 
 static void
 reset_termio()
 {
-#if !defined(MSDOS) && !defined(_Windows)
+#if !defined(MSDOS) && !defined(_WIN32)
 /* reset saved terminal modes */
     if (term_set == 1) {
 #  ifdef SGTTY
@@ -1370,7 +1691,7 @@ reset_termio()
 #  endif			/* not SGTTY */
 	term_set = 0;
     }
-#endif /* not MSDOS && not _Windows */
+#endif /* not MSDOS && not _WIN32 */
 }
 
-#endif /* READLINE && !(HAVE_LIBREADLINE || HAVE_LIBEDITLINE) */
+#endif /* READLINE */
