@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: show.c,v 1.385.2.1 2017/06/10 18:05:14 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - show.c */
 
 /*[
@@ -59,6 +55,7 @@ static char *RCSid() { return RCSid("$Id: show.c,v 1.385.2.1 2017/06/10 18:05:14
 #include "plot3d.h"
 #include "save.h"
 #include "tables.h"
+#include "tabulate.h"
 #include "util.h"
 #include "term_api.h"
 #include "variable.h"
@@ -123,6 +120,7 @@ static void show_palette_colornames __PROTO((void));
 static void show_colorbox __PROTO((void));
 static void show_pointsize __PROTO((void));
 static void show_pointintervalbox __PROTO((void));
+static void show_rgbmax __PROTO((void));
 static void show_encoding __PROTO((void));
 static void show_decimalsign __PROTO((void));
 static void show_fit __PROTO((void));
@@ -160,11 +158,10 @@ static void show_loadpath __PROTO((void));
 static void show_fontpath __PROTO((void));
 static void show_zero __PROTO((void));
 static void show_datafile __PROTO((void));
+static void show_table __PROTO((void));
 static void show_micro __PROTO((void));
 static void show_minus_sign __PROTO((void));
-#ifdef USE_MOUSE
 static void show_mouse __PROTO((void));
-#endif
 static void show_plot __PROTO((void));
 static void show_variables __PROTO((void));
 
@@ -382,6 +379,9 @@ show_command()
     case S_POINTSIZE:
 	show_pointsize();
 	break;
+    case S_RGBMAX:
+	show_rgbmax();
+	break;
     case S_DECIMALSIGN:
 	show_decimalsign();
 	break;
@@ -576,11 +576,12 @@ show_command()
     case S_DATAFILE:
 	show_datafile();
 	break;
-#ifdef USE_MOUSE
+    case S_TABLE:
+	show_table();
+	break;
     case S_MOUSE:
 	show_mouse();
 	break;
-#endif
     case S_PLOT:
 	show_plot();
 #if defined(USE_READLINE)
@@ -797,6 +798,7 @@ show_all()
     show_pm3d();
     show_pointsize();
     show_pointintervalbox();
+    show_rgbmax();
     show_encoding();
     show_decimalsign();
     show_fit();
@@ -1350,6 +1352,11 @@ show_contour()
 		contour_format, clabel_font ? clabel_font : "");
 	fprintf(stderr, "\ton-plot labels placed at segment %d with interval %d\n",
 		clabel_start, clabel_interval);
+	if (contour_firstlinetype > 0)
+		fprintf(stderr, "\tfirst contour linetype will be %d\n", contour_firstlinetype);
+	else
+		fprintf(stderr, "\tfirst contour linetype will be chosen automatically\n");
+	fprintf(stderr, "\tcontour levels will be %ssorted\n", contour_sortlevels ? "" : "un");
     }
 }
 
@@ -1673,6 +1680,8 @@ show_grid()
     fprintf(stderr, "\n\tMinor grid drawn with");
     save_linetype(stderr, &(mgrid_lp), FALSE);
     fputc('\n', stderr);
+    if (grid_vertical_lines)
+	fprintf(stderr, "\tVertical grid lines in 3D plots\n");
     if (polar_grid_angle)
 	fprintf(stderr, "\tGrid radii drawn every %f %s\n",
 		polar_grid_angle / ang2rad,
@@ -2372,8 +2381,6 @@ show_palette()
 	case C_MODEL_RGB: fputs( "RGB\n", stderr ); break;
 	case C_MODEL_HSV: fputs( "HSV\n", stderr ); break;
 	case C_MODEL_CMY: fputs( "CMY\n", stderr ); break;
-	case C_MODEL_YIQ: fputs( "YIQ\n", stderr ); break;
-	case C_MODEL_XYZ: fputs( "XYZ\n", stderr ); break;
 	}
 	fprintf(stderr,"\tgamma is %.4g\n", sm_palette.gamma );
 	return;
@@ -2531,6 +2538,14 @@ show_pointintervalbox()
     fprintf(stderr, "\tpointintervalbox is %g\n", pointintervalbox);
 }
 
+/* process 'show rgbmax' command */
+static void
+show_rgbmax()
+{
+    SHOW_ALL_NL;
+    fprintf(stderr, "\tRGB image color components are in range [0:%g]\n", rgbmax);
+}
+
 
 /* process 'show encoding' command */
 static void
@@ -3103,11 +3118,28 @@ show_datafile()
 	c_token++;
 }
 
-#ifdef USE_MOUSE
+/* process 'show table' command */
+static void
+show_table()
+{
+    char foo[2] = {0,0};
+    foo[0] = (table_sep && *table_sep) ? *table_sep : '\t';
+    SHOW_ALL_NL;
+    if (table_mode)
+	fprintf(stderr, "\ttable mode is on, field separator %s\n",
+		foo[0] == '\t' ? "tab" :
+		foo[0] == ',' ? "comma" :
+		foo[0] == ' ' ? "whitespace" :
+		foo);
+    else
+	fprintf(stderr, "\ttable mode is off\n");
+}
+
 /* process 'show mouse' command */
 static void
 show_mouse()
 {
+#ifdef USE_MOUSE
     SHOW_ALL_NL;
     if (mouse_setting.on) {
 	fprintf(stderr, "\tmouse is on\n");
@@ -3149,8 +3181,10 @@ show_mouse()
     } else {
 	fprintf(stderr, "\tmouse is off\n");
     }
+#else  /* USE_MOUSE */
+    int_warn(NO_CARET, "this copy of gnuplot has no mouse support");
+#endif /* USE_MOUSE */
 }
-#endif
 
 /* process 'show plot' command */
 static void
